#include "r_detectIR.h"

const int DISTRVAL = 40;

void IR_rxInit(void){
	R_CMT1_Create();
	R_ICU_Create();

	R_CMT1_Start();
	R_ICU_IRQ6_Start();
   	itvCnt = 0;
 	FallingFlg = 0;
}

void IR_txInit(void){
	PORTA.PDR.BIT.B4 = 1;
	PORTA.PMR.BIT.B4 = 0;
	R_TMR2_Create();
	R_TMR2_Start();
}

uint8_t IR_detectStart(void){
	if(itvCnt > 1000){
		itvCnt = 0;
	}
	else if(FallingFlg == 1){
		FallingFlg = 0;
		itvCnt = 0;
		return TRUE;
	}
	return FALSE;
}

uint16_t IR_detectData(void){
	uint16_t val;
	if(itvCnt > 1000){
		itvCnt = 0;
		return 1000;
	}
	if(FallingFlg == 1){
		val = itvCnt;
		itvCnt = 0;
		FallingFlg = 0;
		return val;
	}
	return FALSE;
}

uint8_t IR_checkStartBit(uint16_t value,uint16_t compareMin,uint16_t compareMax){
	if(value > compareMin && value < compareMax){
		return TRUE;
	}
	return FALSE;
}

uint8_t IR_checkCustomBit(uint16_t value,uint8_t compareVal){
	if(value == compareVal){
		return TRUE;
	}
	return FALSE;
}

uint8_t IR_convertData(uint16_t * buf, uint8_t startIndex){
	uint8_t value=0, mul=0;
	int i;
	for(i = 0 ; i < 8 ; i++){
		mul = 7 - i;
		value += (buf[startIndex + i] / DISTRVAL) << mul;
	}
	return value;
}

uint8_t * decToBin(uint8_t value){
	static uint8_t arr[8];
	uint8_t i;

	for(i = 0 ; i < 8 ; i++){
		arr[i] = 0;
		arr[i] = ((value >> i) & 0x01);
	}
	return arr;
}

uint16_t * IR_send(uint8_t customCode, uint8_t dataCode){
	int i;
	uint8_t * dataArr;
	static uint16_t returnArr[38] = {0,};

	returnArr[0] = 616;
	returnArr[1] = 308;
	returnArr[18] = 38;
	returnArr[19] = 308;
	returnArr[36] = 38;
	returnArr[37] = 308;

	dataArr = decToBin(customCode);
	for(i = 0 ; i < 8 ; i++){
		returnArr[(i<<1)+2] = 38;
		returnArr[(i<<1)+3] = dataArr[7-i] ? 116 : 38;
	}

	dataArr = decToBin(dataCode);
	for(i = 0 ; i < 8 ; i++){
		returnArr[(i<<1)+20] = 38;
		returnArr[(i<<1)+21] = dataArr[7-i] ? 116 : 38;
	}
	return returnArr;
}
